"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MQTTHub = void 0;
const amqp_client_1 = require("@hive-command/amqp-client");
class MQTTHub {
    constructor(options) {
        this.options = options;
        this.client = new amqp_client_1.MQTTClient({
            user: this.options.user,
            pass: this.options.pass,
            host: this.options.host,
            port: this.options.port,
            exchange: this.options.exchange
        });
    }
    onMessage(msg) {
        var _a, _b, _c, _d, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            // this.channel
            if (!msg)
                return;
            const routingKey = msg === null || msg === void 0 ? void 0 : msg.fields.routingKey;
            const userId = msg === null || msg === void 0 ? void 0 : msg.properties.userId;
            const messageContent = JSON.parse((msg === null || msg === void 0 ? void 0 : msg.content.toString()) || '{error: "No message content"}');
            console.log({ routingKey, userId, messageContent });
            // if(!userId) return console.error("No userId found, private messages not allowed");
            try {
                yield ((_b = (_a = this.options).onMessage) === null || _b === void 0 ? void 0 : _b.call(_a, {
                    routingKey,
                    userId,
                    messageContent
                }));
            }
            catch (e) {
                console.error("Error dealing with message", e);
                return (_d = (_c = this.client) === null || _c === void 0 ? void 0 : _c.channel) === null || _d === void 0 ? void 0 : _d.nack(msg);
            }
            (_f = (_e = this.client) === null || _e === void 0 ? void 0 : _e.channel) === null || _f === void 0 ? void 0 : _f.ack(msg);
        });
    }
    setup() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            // let authSection = this.options.user ? `${this.options.user}:${this.options.pass}` : undefined;
            yield this.client.connect();
            // this.connection = await connect(`amqp://${authSection ? authSection + '@' : ''}${this.options.host}${this.options.port ? `:${this.options.port}` : ''}`)
            // this.channel = await this.connection.createChannel()
            yield ((_b = (_a = this.client) === null || _a === void 0 ? void 0 : _a.channel) === null || _b === void 0 ? void 0 : _b.assertExchange(this.options.exchange, 'topic'));
        });
    }
    subscribe() {
        var _a, _b, _c, _d, _e, _f, _g;
        return __awaiter(this, void 0, void 0, function* () {
            const generatedQueue = yield ((_c = (_b = (_a = this.client) === null || _a === void 0 ? void 0 : _a.channel) === null || _b === void 0 ? void 0 : _b.assertQueue) === null || _c === void 0 ? void 0 : _c.call(_b, ''));
            if (!generatedQueue)
                return;
            yield ((_e = (_d = this.client) === null || _d === void 0 ? void 0 : _d.channel) === null || _e === void 0 ? void 0 : _e.bindQueue(generatedQueue.queue, this.options.exchange, '#'));
            (_g = (_f = this.client) === null || _f === void 0 ? void 0 : _f.channel) === null || _g === void 0 ? void 0 : _g.consume(generatedQueue.queue, this.onMessage.bind(this));
        });
    }
}
exports.MQTTHub = MQTTHub;
